
#!/usr/bin/env python
#
# Script written by Sara Bosticardo
# If you need further assistance, to report any bug or for suggestions
# please contact: sara.bosticardo@univr.it 
# 
#----------------------------------------------------------------------------------------------------------------------------------------------
#
# This script is created to run in a folder organized as follow:
# Main_Analysis_folder/PZ_number/Diffusion
# where "_number" should be the relative number assigned to each subject and the folder "Diffusion" of each subject contains: preprocessed data
#
#----------------------------------------------------------------------------------------------------------------------------------------------
import numpy as np
import os
import shutil

path_fs_file = os.path.join('/...', '...', '...', 'mrtrix3', 'share', 'mrtrix3', 'labelconvert') #path label convert
freesurfer_path = os.path.join('/...' 'freesurfer') #path freesurfer
spath = os.path.join('/...', '...', '...', '...', '...') #path pre-processed data
schedule_path = os.path.join('/usr', 'local', 'fsl', 'etc', 'flirtsch')
freesurfer_folder = os.path.join('/...', '...', '...', '...', '...') #path freesurfer folder of the anatomical images pre-processing 

Subj_list = [f for f in os.listdir(spath) if not f.startswith('.')]
Subj_list.sort()

if len(Subj_list) == 0:
    print(f"Warning! Files not found in {spath}. Please provide the data in the folder.")

for subj in Subj_list:
    print(f"======================= SUBJ-{subj} =======================")

    subj_data_path = os.path.join(spath, subj) 
    path_fs_subj = os.path.join(freesurfer_folder, subj)
    diffusion_path = os.path.join(subj_data_path, 'Diffusion')
    Tractography_upsampling = os.path.join(diffusion_path, 'Tractography_upsampling')
    if not os.path.exists(Tractography_upsampling):
        os.makedirs(Tractography_upsampling) 
    dwi_preproc_bvals = os.path.join(diffusion_path, 'dwi_preproc.bvals')
    dwi_preproc_bvecs = os.path.join(diffusion_path, 'dwi_preproc.bvecs')
    dwi_preproc_B1corr = os.path.join(diffusion_path, 'dwi_preproc_B1corr.nii.gz')
    os.chdir(diffusion_path)
    
    # Upsampling diffusion images
    dwi_preproc_B1corr_upsampl = os.path.join(diffusion_path, 'dwi_preproc_B1corr_upsampl.nii.gz')
    cmd = f"mrgrid -voxel 1 -interp linear {dwi_preproc_B1corr} regrid {dwi_preproc_B1corr_upsampl} -force"
    os.system(cmd)

    #-----------------------------------------------------------------------------------------------------------------
    # perform bet (brain extraction tool) correction
    dwi_preproc_upsaml_bet = os.path.join(diffusion_path, "dwi_preproc_upsaml_bet")
    cmd = f"bet {dwi_preproc_B1corr_upsampl} {dwi_preproc_upsaml_bet} -f 0.2 -m -R -v"
    os.system(cmd)

    #-----------------------------------------------------------------------------------------------------------------------------
    # Creating mean b=0 image
    b0_upsaml = os.path.join(diffusion_path, "b0_upsaml.nii.gz")
    cmd = f"dwiextract -bzero {dwi_preproc_B1corr_upsampl} {b0_upsaml} -fslgrad {dwi_preproc_bvecs} {dwi_preproc_bvals} -force -nthreads 14"
    os.system(cmd)

    # Create a single b0 composed of the averages of the previous b0s 
    b0mean_upsaml = os.path.join(diffusion_path, "b0mean_upsaml.nii.gz")    
    cmd = f"fslmaths {b0_upsaml} -Tmean {b0mean_upsaml}"
    os.system(cmd)
    # Multiply by the BET to remove the skull
    dwi_preproc_upsaml_bet_mask = os.path.join(diffusion_path, "dwi_preproc_upsaml_bet_mask.nii.gz")    
    cmd = f"fslmaths {b0mean_upsaml} -mul {dwi_preproc_upsaml_bet_mask} {b0mean_upsaml}"
    os.system(cmd)
    #-----------------------------------------------------------------------------------------------------------------------------
    # Generate 5tt image suitable for Anatomically-Constrained Tractography (ACT)
    cinqueTT = os.path.join(Tractography_upsampling, '5TT')
    if not os.path.exists(cinqueTT):
        os.makedirs(cinqueTT)

    # Converting the T1 brain found with FreeSurfer in nifty and creating the brain mask
    T1 = os.path.join(subj_data_path, 'T1')
    if not os.path.exists(T1):
        os.makedirs(T1)

    mri = os.path.join(path_fs_subj, 'mri')
    brain = os.path.join(mri, 'brain.mgz')
    T1_brain_mask_FS = os.path.join(T1, "T1_brain_mask_FS.nii.gz")    
    # convert from mgz to nii.gz and change the orientation
    cmd = f"mri_convert {brain} {T1_brain_mask_FS} --out_orientation LAS"
    os.system(cmd)
    
    orig_nu = os.path.join(mri, 'orig_nu.mgz')
    T1_nucorr_FS = os.path.join(T1, "T1_nucorr_FS.nii.gz")    
    # convert from mgz to nii.gz and change the orientation
    cmd = f"mri_convert {orig_nu} {T1_nucorr_FS} --out_orientation LAS"
    os.system(cmd)  

    #Binarize the T1_brain_mask
    cmd = f"fslmaths {T1_brain_mask_FS} -bin {T1_brain_mask_FS}"
    os.system(cmd)    

    T1_T1_brain_FS = os.path.join(T1, "T1_brain_FS.nii.gz" )
    cmd = f"fslmaths {T1_nucorr_FS} -mul {T1_brain_mask_FS} {T1_T1_brain_FS}"
    os.system(cmd) 

    TT_T1_brain_FS = os.path.join(cinqueTT, "T1_brain_FS.nii.gz")
    shutil.copyfile(T1_T1_brain_FS, TT_T1_brain_FS)

    # 5tt generation
    os.chdir(Tractography_upsampling)
    five_tissue_tipe_tosplit = os.path.join(cinqueTT, f"5TT_tosplit.nii.gz")    
    aseg_in_rawavg_mgz = os.path.join(path_fs_subj, 'mri', 'aseg-in-rawavg.mgz')
    cmd = f"5ttgen freesurfer {aseg_in_rawavg_mgz} {five_tissue_tipe_tosplit} -nocrop -sgm_amyg_hipp -nocleanup -force"
    os.system(cmd)  

    cmd = f"fslsplit {five_tissue_tipe_tosplit} split"
    os.system(cmd)

    split00000 = os.path.join(Tractography_upsampling, 'split0000.nii.gz')
    split00001 = os.path.join(Tractography_upsampling, 'split0001.nii.gz')
    split00002 = os.path.join(Tractography_upsampling, 'split0002.nii.gz')
    split00003 = os.path.join(Tractography_upsampling, 'split0003.nii.gz')
    split00004 = os.path.join(Tractography_upsampling, 'split0004.nii.gz')

    cmd = f"fslmaths {split00002} -add {split00004} {split00002}"
    os.system(cmd)

    cmd = f"fslmaths {split00004} -mul 0 {split00004} -odt float"
    os.system(cmd)

    five_tissue_tipe = os.path.join(cinqueTT, f"5TT.nii.gz")  
    cmd = f"fslmerge -t {five_tissue_tipe} {split00000} {split00001} {split00002} {split00003} {split00004}"
    os.system(cmd)
    os.chdir('../..')

    # Generate WM-GM interface if needed (not necessary if we use -seed_dinamic in tckgen)
    five_tissue_tipe_mask_mif = os.path.join(cinqueTT, f"5TT_mask.mif")  
    cmd = f"5tt2gmwmi {five_tissue_tipe} {five_tissue_tipe_mask_mif} -force"
    os.system(cmd)

    five_tissue_tipe_mask = os.path.join(cinqueTT, f"5TT_mask.nii.gz") 
    cmd = f"mrconvert {five_tissue_tipe_mask_mif} {five_tissue_tipe_mask}"
    os.system(cmd)

    os.remove(split00000)
    os.remove(split00001)
    os.remove(split00002)
    os.remove(split00003)
    os.remove(split00004)

    #------------------------------------------------------------------------------------------------------
    # Registering T1 brain of FreeSurfer with b0
    T1_on_DTI = os.path.join(Tractography_upsampling, 'T1_on_DTI')
    if not os.path.exists(T1_on_DTI):
        os.makedirs(T1_on_DTI) 

    cartelle = os.listdir(Tractography_upsampling)
    folder_5ttgen = [ c for c in cartelle if '5ttgen' in c][0]
    path_5ttgen = os.path.join(Tractography_upsampling, folder_5ttgen)
    whitematter_mrtrix = os.path.join(path_5ttgen, 'wm.mif')
    whitematter_freesurfer = os.path.join(cinqueTT, f"T1_wm_fsurfer.nii.gz")
    
    # Convert wm.mif in wm.nii.gz
    cmd = f"mrconvert {whitematter_mrtrix} {whitematter_freesurfer} -force -nthreads 14"
    os.system(cmd)

    tmp_mat_fsurfer_fsl = os.path.join(cinqueTT, 'tmp_mat_fsurfer_fsl.mat')
    cmd = f"flirt -v -in {whitematter_freesurfer} -ref {TT_T1_brain_FS} -dof 6 -omat {tmp_mat_fsurfer_fsl}"
    os.system(cmd)

    whitematter_fsl = os.path.join(cinqueTT, f"T1_wm.nii.gz")
    cmd = f"flirt -v -applyxfm -init {tmp_mat_fsurfer_fsl} -in {whitematter_freesurfer} -ref {TT_T1_brain_FS} -out {whitematter_fsl}"
    os.system(cmd)

    #Preliminary FLIRT registration
    tmp_map = os.path.join(T1_on_DTI, 'tmp.mat')
    cmd = f"flirt -v -in {b0mean_upsaml} -ref {TT_T1_brain_FS} -dof 6 -omat {tmp_map}"
    os.system(cmd)

    prereg_output_matrix = os.path.join(T1_on_DTI, 'diff_to_structural_bbr.mat')
    schedule = os.path.join(schedule_path, 'bbr.sch')
    segmentation_result = os.path.join(T1_on_DTI, 'diff_to_structural_bbr.nii.gz')
    
    # refine the registration 
    cmd = f"flirt -v -in {b0mean_upsaml} -ref {TT_T1_brain_FS} -dof 6 -cost bbr -wmseg {whitematter_fsl} -init {tmp_map} -omat {prereg_output_matrix} -out {segmentation_result} -schedule {schedule}"
    os.system(cmd)

  
    diff_to_structural_bbr_inverse = os.path.join(T1_on_DTI, 'diff_to_structural_bbr_inverse.mat')    
    # Invert the previous registration
    cmd = f"convert_xfm -omat {diff_to_structural_bbr_inverse} -inverse {prereg_output_matrix}"
    os.system(cmd)

    T1_to_b0mean_FSLbbr = os.path.join(T1_on_DTI, "T1_to_b0mean_FSLbbr.nii.gz")
    # Applying the inverse transformation to T1
    cmd = f"flirt -v -applyxfm -init {diff_to_structural_bbr_inverse} -in {TT_T1_brain_FS} -ref {b0mean_upsaml} -out {T1_to_b0mean_FSLbbr}"
    os.system(cmd)

    WM_to_b0 = os.path.join(T1_on_DTI, "WM_to_b0.nii.gz")
    #Applying the inverse transformation to T1_WM
    cmd = f"flirt -v -applyxfm -init {diff_to_structural_bbr_inverse} -in {whitematter_fsl} -ref {b0mean_upsaml} -out {WM_to_b0}"
    os.system(cmd)

    WM_mask_to_b0 = os.path.join(T1_on_DTI, "WM_mask_to_b0.nii.gz")
    cmd = f"fslmaths {WM_to_b0} -bin {WM_mask_to_b0}"
    os.system(cmd)

    cmd = f"fslmaths {WM_mask_to_b0} -mul {dwi_preproc_upsaml_bet_mask} {WM_mask_to_b0}"
    os.system(cmd)

    WM_to_b0_bin = os.path.join(T1_on_DTI, "WM_to_b0_bin.nii.gz")
    cmd = f"fslmaths {WM_to_b0} -bin {WM_to_b0_bin}"
    os.system(cmd)

    diff_to_structural_bbr_mrtrixformat = os.path.join(cinqueTT, "diff_to_structural_bbr-mrtrixformat.txt") 
    cmd = f"transformconvert {prereg_output_matrix} {b0mean_upsaml} {TT_T1_brain_FS} flirt_import {diff_to_structural_bbr_mrtrixformat} -force -nthreads 14"
    os.system(cmd)   

    r5TT = os.path.join(cinqueTT, "r5TT.nii.gz")
    cmd = f"mrtransform {five_tissue_tipe} {r5TT} -linear {diff_to_structural_bbr_mrtrixformat} -inverse -force -nthreads 14"
    os.system(cmd)    

    r5TT_mask = os.path.join(cinqueTT, "r5TT_mask.nii.gz")
    cmd = f"mrtransform {five_tissue_tipe_mask} {r5TT_mask} -linear {diff_to_structural_bbr_mrtrixformat} -inverse -force -nthreads 14"
    os.system(cmd)

    #-----------------------------------------------------------------------------------------------------------------------------
    # Computing the response functions with multi shell multi tissue
    CSD = os.path.join(Tractography_upsampling, 'CSD')
    if not os.path.exists(CSD):
        os.makedirs(CSD) 

    csd_response_wm = os.path.join(CSD, "response_wm.txt")
    csd_response_gm = os.path.join(CSD, "response_gm.txt")
    csd_response_csf = os.path.join(CSD, "response_csf.txt")
    RF_voxels = os.path.join(CSD, "RF_voxels.mif")
    
    cmd = f"dwi2response msmt_5tt {dwi_preproc_B1corr_upsampl} {r5TT} {csd_response_wm} {csd_response_gm} {csd_response_csf} -fslgrad {dwi_preproc_bvecs} {dwi_preproc_bvals} -mask {dwi_preproc_upsaml_bet_mask} -voxels {RF_voxels} -force -nthreads 14"
    os.system(cmd)

    #-----------------------------------------------------------------------------------------------------------------------------
    # Computing the FOD with multi shells multi tissue 
    FOD = os.path.join(Tractography_upsampling, 'FOD')
    if not os.path.exists(FOD) :
        os.makedirs(FOD)

    fod_WM_FODs = os.path.join(FOD, "WM_FODs.mif")
    fod_GM_FODs = os.path.join(FOD, "GM_FODs.mif")
    fod_CSF_FODs = os.path.join(FOD, "CSF.mif")
    
    cmd = f"dwi2fod msmt_csd {dwi_preproc_B1corr_upsampl} {csd_response_wm} {fod_WM_FODs} {csd_response_gm} {fod_GM_FODs} {csd_response_csf} {fod_CSF_FODs} -fslgrad {dwi_preproc_bvecs} {dwi_preproc_bvals} -mask {dwi_preproc_upsaml_bet_mask} -force -nthreads 14"
    os.system(cmd)

    #-----------------------------------------------------------------------------------------------------------------------------
    fod_WM_norm = os.path.join(FOD, "WM_FODs_norm.nii.gz")
    fod_GM_norm = os.path.join(FOD, "GM_norm.nii.gz")
    fod_CSF_norm = os.path.join(FOD, "CSF_norm.nii.gz")
    mtnorm = os.path.join(FOD, "mtnormalise_norm.nii.gz")
    mtnorm_mask = os.path.join(FOD, "mtnormalise_mask.nii.gz")
    
    cmd = f"mtnormalise {fod_WM_FODs} {fod_WM_norm} {fod_GM_FODs} {fod_GM_norm} {fod_CSF_FODs} {fod_CSF_norm} -mask {dwi_preproc_upsaml_bet_mask} -check_norm {mtnorm} -check_mask {mtnorm_mask} -force"
    os.system(cmd)

    #-----------------------------------------------------------------------------------------------------------------------------
    # Tractography reconstruction

    iFOD_tck = os.path.join(Tractography_upsampling, f"iFOD2_ACT_3M.tck")
    cmd = f"tckgen {fod_WM_norm} {iFOD_tck} -act {r5TT} -backtrack -seed_gmwmi {r5TT_mask} -select 3M -maxlength 250 -minlength 5 -cutoff 0.05 -power 3 -angle 30 -force -nthreads 14"
    os.system(cmd)

    #-----------------------------------------------------------------------------------------------------------------------------
    # Creating the connectome using the number of streamlines 
    aparc_aseg = os.path.join(mri, 'aparc+aseg.mgz')
    t1_aparc_aseg = os.path.join(T1, "aparc+aseg.nii.gz")
    
    cmd = f"mri_convert {aparc_aseg} {t1_aparc_aseg} --out_orientation LAS"
    os.system(cmd)

    brainstemSsLabels = os.path.join(mri, 'brainstemSsLabels.v10.FSvoxelSpace.mgz')
    t1_brainstemSsLabels = os.path.join(T1, "T1_brain_FS_bstem.nii.gz")
    cmd = f"mri_convert {brainstemSsLabels} {t1_brainstemSsLabels} --out_orientation LAS"
    os.system(cmd)

    #Only if you need the medulla node
    T1_brain_FS_bstem_binv = os.path.join(T1, "T1_brain_FS_bstem_binv.nii.gz")
    
    cmd = f"fslmaths {t1_aparc_aseg} -uthr 16 -thr 16 -add {t1_brainstemSsLabels} -binv {T1_brain_FS_bstem_binv}"
    os.system(cmd)

    aparc_aseg_FS_nobstem = os.path.join(T1, "aparc+aseg_FS_nobstem.nii.gz")
    cmd = f"fslmaths {t1_aparc_aseg} -mul {T1_brain_FS_bstem_binv} {aparc_aseg_FS_nobstem}"
    os.system(cmd)

    T1_brain_FS_medulla = os.path.join(T1, "T1_brain_FS_medulla.nii.gz")
    cmd = f"fslmaths {t1_brainstemSsLabels} -uthr 175 -thr 175 -bin {T1_brain_FS_medulla}"
    os.system(cmd)

    cmd = f"fslmaths {T1_brain_FS_medulla} -mul 16 {T1_brain_FS_medulla}"
    os.system(cmd)

    aparc_aseg_medulla = os.path.join(T1, "aparc+aseg_medulla.nii.gz")
    cmd = f"fslmaths {aparc_aseg_FS_nobstem} -add {T1_brain_FS_medulla} {aparc_aseg_medulla}"
    os.system(cmd)

    Connectome_upsampl = os.path.join(subj_data_path, 'Connectome_upsampl')
    if not os.path.exists(Connectome_upsampl):
        os.makedirs(Connectome_upsampl)

    Connectome = os.path.join(subj_data_path, 'Connectome')
    if not os.path.exists(Connectome) :
        os.makedirs(Connectome)

    FreesurferColorLUT = os.path.join(freesurfer_path, 'FreeSurferColorLUT.txt')
    fs_default_Bstem = os.path.join(path_fs_file, 'fs_default_Bstem.txt')
    nodes = os.path.join(Connectome, "nodes.nii.gz")
    
    cmd = f"labelconvert {aparc_aseg_medulla} {FreesurferColorLUT} {fs_default_Bstem} {nodes} -force -nthreads 14"
    os.system(cmd)

    nodes_fixSGM = os.path.join(Connectome, "nodes_fixSGM.nii.gz")
    os.chdir(subj_data_path)
    
    cmd = f"labelsgmfix {nodes} {T1_T1_brain_FS} {fs_default_Bstem} {nodes_fixSGM} -sgm_amyg_hipp -premasked -force -nthreads 14"
    os.system(cmd)

    nodes_fixSGM_tob0 = os.path.join(Connectome_upsampl, "nodes_fixSGM_tob0.nii.gz")
    cmd = f"flirt -v -interp nearestneighbour -applyxfm -init {diff_to_structural_bbr_inverse} -in {nodes_fixSGM} -ref {b0mean_upsaml} -out {nodes_fixSGM_tob0}"
    os.system(cmd)

    assignment_3M_iFOD2 = os.path.join(Connectome_upsampl, f"iFOD2_ACT_3M_assignment.txt")
    connectome_3M_iFOD2 = os.path.join(Connectome_upsampl, f"iFOD2_ACT_3M_connectome.csv")
    cmd = f"tck2connectome -symmetric {iFOD_tck} -out_assignments {assignment_3M_iFOD2} {nodes_fixSGM_tob0} {connectome_3M_iFOD2} -assignment_radial_search 3 -force -nthreads 14"
    os.system(cmd)

    fibers_connecting_iFOD2_tck = os.path.join(Tractography_upsampling, "iFOD2_ACT_3M_connecting.tck")
    cmd = f"connectome2tck -force {iFOD_tck} {assignment_3M_iFOD2} {fibers_connecting_iFOD2_tck} -files single -keep_self -nodes 1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85 -exclusive -nthreads 14"
    os.system(cmd)