
#!/usr/bin/env python
#
# Script written by Sara Bosticardo
# If you need further assistance, to report any bug or for suggestions
# please contact: sara.bosticardo@univr.it 
# 
#----------------------------------------------------------------------------------------------------------------------------------------------
#
# This script is created to run in a folder organized as follow:
# Main_Analysis_folder/PZ_number/Diffusion
# where "_number" should be the relative number assigned to each subject and the folder "Diffusion" of each subject contains: preprocessed data
#
#----------------------------------------------------------------------------------------------------------------------------------------------
import numpy as np
import os
import shutil

path_fs_file = os.path.join('/...', '...', '...', 'mrtrix3', 'share', 'mrtrix3', 'labelconvert') #path label convert
freesurfer_path = os.path.join('/...' 'freesurfer') #path freesurfer
spath = os.path.join('/...', '...', '...', '...', '...') #path pre-processed data
schedule_path = os.path.join('/usr', 'local', 'fsl', 'etc', 'flirtsch')
freesurfer_folder = os.path.join('/...', '...', '...', '...', '...') #path freesurfer folder of the anatomical images pre-processing 

Subj_list = [f for f in os.listdir(spath) if not f.startswith('.')]
Subj_list.sort()

if len(Subj_list) == 0:
    print(f"Warning! Files not found in {spath}. Please provide the data in the folder.")

for subj in Subj_list:
    print(f"======================= SUBJ-{subj} =======================")

    subj_data_path = os.path.join(spath, subj) 
    path_fs_subj = os.path.join(freesurfer_folder, subj)
    diffusion_path = os.path.join(subj_data_path, 'Diffusion')
    Tractography_upsampling = os.path.join(diffusion_path, 'Tractography_upsampling')

    commit_subj = os.path.join(diffusion_path, 'COMMIT_SZB')
    if not os.path.exists(commit_subj):
        os.makedirs(commit_subj)
    os.chdir(commit_subj) 

    Tractography = os.path.join(diffusion_path, 'Tractography_upsampling')
    FOD_folder = os.path.join(Tractography, 'FOD')
    T1_on_DTI = os.path.join(Tractography, 'T1_on_DTI')
    WM_FOD_norm = os.path.join(FOD_folder, 'WM_FODs_norm.nii.gz')

    tractogram = os.path.join(Tractography, 'iFOD2_ACT_3M_connecting.tck')
    T1_wm_mask = os.path.join(T1_on_DTI, 'WM_mask_to_b0.nii.gz')
    #dwi_preproc_B1corr = os.path.join(hdd_subj, 'dwi_preproc_upsaml_bet.nii.gz')
    dwi_preproc_B1corr = os.path.join(diffusion_path, 'dwi_preproc_B1corr_upsampl.nii.gz')
    bvals = os.path.join(diffusion_path, 'dwi_preproc.bvals')
    bvecs = os.path.join(diffusion_path, 'dwi_preproc.bvecs')
    Connectome_upsampl = os.path.join(subj_data_path, 'Connectome_upsampl')
    nodes = os.path.join(Connectome_upsampl, "nodes_fixSGM_tob0.nii.gz")

    # Setting the input data

    bundle = os.path.join(commit_subj, 'bundles')
    if not os.path.exists(bundle):
        os.makedirs(bundle) 

    assignment_nos = os.path.join(Connectome_upsampl, f"iFOD2_ACT_3M_assignment.txt")
    connectome_nos = os.path.join(Connectome_upsampl, f"iFOD2_ACT_3M_connectome.csv")
    os.system( f'tck2connectome -force -assignment_radial_search 3 -out_assignments {assignment_nos} {tractogram} {nodes} {connectome_nos}' )

    os.chdir(bundle)
    os.system( f'connectome2tck -force -nthreads 0 -exclusive -files per_edge -keep_self {tractogram} {assignment_nos} bundle_' )

    C = np.loadtxt( connectome_nos, delimiter=',' ) # NB: change 'delimiter' to suits your needs
    CMD = 'tckedit -force -nthreads 1'
    for i in range(C.shape[0]):
        for j in range(i,C.shape[0]):
            if C[i,j] > 0 :
                CMD += f' bundle_%d-%d.tck' %(i+1,j+1)
    os.system( CMD + f' {commit_subj}/fibers_connecting_filtered.tck' )
    filtered_tracto = os.path.join(commit_subj, 'fibers_connecting_filtered.tck')

    peaks = os.path.join(FOD_folder, 'peaks.nii.gz')
    cmd = f"sh2peaks {WM_FOD_norm} {peaks} -force"
    os.system(cmd)

    os.chdir(commit_subj)
    from commit import trk2dictionary
    trk2dictionary.run(
        ndirs = 500,
        filename_tractogram = filtered_tracto,
        filename_peaks      = peaks,
        filename_mask       = T1_wm_mask,
        path_out            = 'COMMIT',
        fiber_shift         = 0.5,
        peaks_use_affine    = True
    )

    scheme = os.path.join(FOD_folder, 'DWI.scheme')
    amico.util.fsl2scheme(bvals, bvecs, scheme)

    # Setting parameters  
    mit = commit.Evaluation( '.', '.' )   
    mit.load_data( dwi_preproc_B1corr, scheme )

    # Set model and generate the kernel
    mit.set_model( 'StickZeppelinBall' )
    d_par       = 1.7E-3             # Parallel diffusivity [mm^2/s]
    d_perps_zep = [ 0.51E-3 ]        # Perpendicular diffusivity(s) [mm^2/s]
    d_isos      = [ 1.7E-3, 3.0E-3 ] # Isotropic diffusivity(s) [mm^2/s]
    mit.model.set( d_par, d_perps_zep, d_isos )
    mit.generate_kernels( regenerate=True, ndirs = 500 )
    mit.load_kernels()

    # Load dictionary and buid the operator
    mit.load_dictionary( 'COMMIT' )
    mit.set_threads()
    mit.build_operator()

    # fitting
    mit.fit( tol_fun=1e-3, max_iter=1000, verbose=True ) 
    mit.save_results(path_suffix = "_COMMIT1")
    x_nnls, _, _ = mit.get_coeffs( get_normalized=False )

    C = np.loadtxt( connectome_nos, delimiter=',' )
    C = np.triu( C ) # be sure to get only the upper-triangular part of the matrix
    group_size = C[C>0].astype(np.int32)

    tmp = np.insert( np.cumsum(group_size), 0 , 0)
    group_idx = np.array( [np.arange(tmp[i],tmp[i+1]) for i in range(len(tmp)-1)] )

    group_w = np.empty_like( group_size, dtype=np.float64 )
    for k in range(group_size.size) :
        group_w[k] = np.sqrt(group_size[k]) / ( np.linalg.norm(x_nnls[group_idx[k]]) + 1e-12 )

    reg_lambda = 5e-4 #change to suit your needs

    prior_on_bundles = commit.solvers.init_regularisation(
        mit,
        regnorms    = [commit.solvers.group_sparsity, commit.solvers.non_negative, commit.solvers.non_negative],
        structureIC = group_idx,
        weightsIC   = group_w,
        lambdas     = [reg_lambda, 0.0, 0.0]
    )

    mit.fit( tol_fun=1e-3, max_iter=1000, regularisation=prior_on_bundles, verbose=False )
    mit.save_results( path_suffix="_COMMIT2" )

    # extracting non-zero streamlines

    Results = os.path.join(commit_subj, 'COMMIT', 'Results_StickZeppelinBall_COMMIT2')

    streamline_weights = os.path.join(Results, 'streamline_weights.txt')

    mitX_filtered_txt = os.path.join(Results, 'mitX_filtered_2.txt')
    mitX_filtered_tck = os.path.join(Results, 'mitX_filtered_2.tck')

    cmd = f"tckedit -minweight 0.000000000001 -tck_weights_in {streamline_weights} -tck_weights_out {mitX_filtered_txt} {filtered_tracto} {mitX_filtered_tck}"
    os.system(cmd)

    COMMIT_connectomes = os.path.join(commit_subj, 'COMMIT2_connectomes')
    if not os.path.exists(COMMIT_connectomes):
        os.makedirs(COMMIT_connectomes) 

    mitX_filtered_assignments = os.path.join(COMMIT_connectomes, f'mitX2_filtered_assignments.txt')
    mitX_filtered_connectome = os.path.join(COMMIT_connectomes,f'mitX2_filtered_connectome.csv')

    cmd = f"tck2connectome -symmetric {mitX_filtered_tck} -out_assignments {mitX_filtered_assignments} {nodes} {mitX_filtered_connectome} -assignment_radial_search 2" 
    os.system(cmd)     
