#!/usr/bin/env python
#
# Script written by Sara Bosticardo
# If you need further assistance, to report any bug or for suggestions
# please contact: sara.bosticardo@univr.it 
# 
#----------------------------------------------------------------------------------------------------------------------------------------------
#
# This script is created to run in a folder organized as follow:
# Main_Analysis_folder/PZ_number/Diffusion
# where "_number" should be the relative number assigned to each subject and the folder "Diffusion" of each subject contains: preprocessed data
#
#----------------------------------------------------------------------------------------------------------------------------------------------
import numpy as np
import os
import shutil

spath = os.path.join('/...', '...', '...', '...', '...') # main analysis folder

Subj_list = [f for f in os.listdir(spath) if not f.startswith('.')]
Subj_list.sort()

for subj in Subj_list:

    path_subj_analysis = os.path.join(spath, subj, 'Diffusion')
    os.chdir(path_subj_analysis)
    print(path_subj_analysis)
    tensor_fitting = os.path.join(path_subj_analysis, 'Tensor_fitting')
    smt_path = os.path.join(tensor_fitting, 'SMT')
    dwi_preproc_B1corr = os.path.join(path_subj_analysis, 'dwi_preproc_B1corr.nii.gz')
    dwi_preproc_bvals = os.path.join(path_subj_analysis, 'dwi_preproc.bvals')
    dwi_preproc_bvecs = os.path.join(path_subj_analysis, 'dwi_preproc.bvecs')

    if not os.path.exists(smt_path) :
        os.mkdir(smt_path)

    data_UpTo_b2000 = os.path.join(tensor_fitting, 'data_UpTo_b2000.nii.gz')
    data_UpTo_b2000_bvecs = os.path.join(tensor_fitting, 'data_UpTo_b2000.bvecs')
    data_UpTo_b2000_bvals = os.path.join(tensor_fitting, 'data_UpTo_b2000.bvals')
    cmd = f'dwiextract -shells 0,700,1000,2000 {dwi_preproc_B1corr} {data_UpTo_b2000} -fslgrad {dwi_preproc_bvecs} {dwi_preproc_bvals} -export_grad_fsl {data_UpTo_b2000_bvecs} {data_UpTo_b2000_bvals}'
    os.system(cmd) 

    data_UpTo_b2000_bet = os.path.join(tensor_fitting, 'data_UpTo_b2000_bet')
    cmd = f'bet {data_UpTo_b2000} {data_UpTo_b2000_bet} -f 0.2 -R -m -v'
    os.system(cmd)
    
    os.chdir(path_subj_analysis)
    cmd = 'fitmcmicro --bvals Tensor_fitting/data_UpTo_b2000.bvals --bvecs Tensor_fitting/data_UpTo_b2000.bvecs --mask Tensor_fitting/data_UpTo_b2000_bet_mask Tensor_fitting/data_UpTo_b2000.nii.gz SMT/SMT_results_{}.nii'
    os.system(cmd)

    smt_upsampling = os.path.join(smt_path, 'SMT_upsampling')
    os.makedirs(smt_upsampling, exist_ok=True)
    
    os.chdir(tensor_fitting)
    cmd = f'mrresize -voxel 1 -interp linear SMT/SMT_results_intra.nii SMT/SMT_upsampling/SMT_results_intra_upsampling.nii.gz'
    os.system(cmd)

    print("The script is terminated without errors")


