#!/usr/bin/env python
#
# Script written by Sara Bosticardo
# If you need further assistance, to report any bug or for suggestions
# please contact: sara.bosticardo@univr.it 
# 
#----------------------------------------------------------------------------------------------------------------------------------------------
#
# This script is created to run in a folder organized as follow:
# Main_Analysis_folder/PZ_number/Diffusion
# where "_number" should be the relative number assigned to each subject and the folder "Diffusion" of each subject contains: preprocessed data, and 
# path_smt contains the INVF map computed with SMT
#
#----------------------------------------------------------------------------------------------------------------------------------------------
import numpy as np
import pickle
import amico
import os

spath = os.path.join('/...', '...', '...', '...', '...') #path pre-processed data
subj_list = [f for f in os.listdir(spath) if not f.startswith('.')]
subj_list.sort()
spath = os.path.join()
for subj in subj_list:
    path_subj_analysis = os.path.join(spath, subj, 'Diffusion')

    os.chdir(path_subj_analysis)

    Tractography = os.path.join(path_subj_analysis, 'Tractography_upsampling')
    tractogram = os.path.join(path_subj_analysis, 'COMMIT_SZB', 'COMMIT', 'Results_StickZeppelinBall_COMMIT2', 'mitX_filtered_2.tck')
    nodes = os.path.join(path_subj_analysis, 'Connectome_upsampl', 'nodes_fixSGM_tob0.nii.gz')
    path_smt = os.path.join(path_subj_analysis, 'Tensor_fitting', 'SMT', 'SMT_upsampling', 'SMT_results_intra_upsampling.nii.gz')

    os.chdir(path_subj_analysis)

    tractometry_folder = os.path.join(path_subj_analysis, 'Tractometry_intra')
    if not os.path.exists( tractometry_folder ) :
        os.makedirs(tractometry_folder)

        intra_mean_tractometry_txt = os.path.join(tractometry_folder, 'INTRA_tractometry_mean.txt')
        cmd = f'tcksample {tractogram} {path_smt} {intra_mean_tractometry_txt} -stat_tck mean'
        os.system(cmd)
        intra_mean_tractometry_csv = os.path.join(tractometry_folder, 'INTRA_tractometry_mean.csv')
        cmd = f'tck2connectome {tractogram} {nodes} {intra_mean_tractometry_csv} -scale_file {intra_mean_tractometry_txt} -stat_edge mean -symmetric'
        os.system(cmd)

        intra_median_tractometry_txt = os.path.join(tractometry_folder, 'INTRA_tractometry_median.txt')
        cmd = f'tcksample {tractogram} {path_smt} {intra_median_tractometry_txt} -stat_tck median'
        os.system(cmd)
        intra_median_tractometry_csv = os.path.join(tractometry_folder, 'INTRA_tractometry_median_intra.csv')
        cmd = f'tck2connectome {tractogram} {nodes} {intra_median_tractometry_csv} -scale_file {intra_median_tractometry_txt} -stat_edge mean -symmetric'
        os.system(cmd)


        