#!/usr/bin/env python
#
# Script written by Sara Bosticardo
# If you need further assistance, to report any bug or for suggestions
# please contact: sara.bosticardo@univr.it 
# 
#----------------------------------------------------------------------------------------------------------------------------------------------
#
# This script is created to run in a folder organized as follow:
# Main_Analysis_folder/PZ_number/Diffusion
# where "_number" should be the relative number assigned to each subject and the folder "Diffusion" of each subject contains: preprocessed data, and 
# path_mvf contains the MTsat map already multiplied by the alfa constant and registered to dMRI images
#
#----------------------------------------------------------------------------------------------------------------------------------------------
import numpy as np
import pickle
import amico
import os

spath = os.path.join('/...', '...', '...', '...', '...') #path pre-processed data
path_mvf = os.path.join('...', '...', '...', '...', 'MVF') #path MVF images
subj_list = [f for f in os.listdir(spath) if not f.startswith('.')]
subj_list.sort()
spath = os.path.join()

for subj in subj_list:
    path_subj_analysis = os.path.join(spath, subj, 'Diffusion')
    Subj_path_mvf = os.path.join(path_mvf, subj)

    os.chdir(path_subj_analysis)

    Tractography = os.path.join(path_subj_analysis, 'Tractography_upsampling')
    tractogram = os.path.join(path_subj_analysis, 'COMMIT_SZB', 'COMMIT', 'Results_StickZeppelinBall_COMMIT2', 'mitX_filtered_2.tck')
    mielina_reg_on_b0 = os.path.join(Subj_path_mvf, 'MVF_on_b0.nii.gz')
    nodes = os.path.join(path_subj_analysis, 'Connectome_upsampl', 'nodes_fixSGM_tob0.nii.gz')

    tractometry_folder = os.path.join(path_subj_analysis, 'Tractometry_Mielina_MVF')
    if not os.path.exists( tractometry_folder ) :
        os.makedirs(tractometry_folder)

        mwf_mean_tractometry_txt = os.path.join(tractometry_folder, 'MVF_tractometry_mean.txt')
        cmd = f'tcksample {tractogram} {mielina_reg_on_b0} {mwf_mean_tractometry_txt} -stat_tck mean'
        os.system(cmd)
        mwf_mean_tractometry_csv = os.path.join(tractometry_folder, 'MVF_tractometry_mean.csv')
        cmd = f'tck2connectome {tractogram} {nodes} {mwf_mean_tractometry_csv} -scale_file {mwf_mean_tractometry_txt} -stat_edge mean -symmetric'
        os.system(cmd)

        mwf_median_tractometry_txt = os.path.join(tractometry_folder, 'MVF_tractometry_median.txt')
        cmd = f'tcksample {tractogram} {mielina_reg_on_b0} {mwf_median_tractometry_txt} -stat_tck median'
        os.system(cmd)
        mwf_median_tractometry_csv = os.path.join(tractometry_folder, 'MVF_tractometry_median.csv')
        cmd = f'tck2connectome {tractogram} {nodes} {mwf_median_tractometry_csv} -scale_file {mwf_median_tractometry_txt} -stat_edge mean -symmetric'
        os.system(cmd)

        