#!/usr/bin/env python
#
# Script written by Sara Bosticardo
# If you need further assistance, to report any bug or for suggestions
# please contact: sara.bosticardo@univr.it 
# 
#----------------------------------------------------------------------------------------------------------------------------------------------
#
# This script is created to run in a folder organized as follow:
# Main_Analysis_folder/PZ_number/Diffusion
# where "_number" should be the relative number assigned to each subject and the folder "Diffusion" of each subject contains: preprocessed data
#
#----------------------------------------------------------------------------------------------------------------------------------------------
import numpy as np
import os
import shutil
import bct
import csv
import pandas as pd

gpath = os.path.join('...', '...', '...', '...')
spath = os.path.join('...', '...', '...', '...', '...')
subj_list = [f for f in os.listdir(spath) if not f.startswith('.')]
subj_list.sort()
s = 0

data = pd.DataFrame(columns=["Subj", "strength", "efficiency", "modularity"])

if len(subj_list) == 0:
    print(f"Warning! Files not found in {spath}. Please provide the data in the folder.")

for subj in subj_list:
    print(f"======================= SUBJ-{subj} =======================")

    connectome_file = os.path.join(spath, subj, '...', '...', 'xxx.csv')
    connectome = np.loadtxt(connectome_file, delimiter = ',')

    density = bct.density_und(connectome)[0]
    efficiency = bct.efficiency_wei(connectome)
    modularity = bct.modularity_und(connectome)[1]
    clustering_coeff = np.mean(bct.clustering_coef_bu(connectome))
    mean_strength = np.mean(bct.strengths_und(connectome))

    data.loc[s,'Subj'] = subj
    data.loc[s,'efficiency'] = efficiency
    data.loc[s, 'modularity'] = modularity
    data.loc[s, 'strength'] = mean_strength

    s +=1

data.to_csv(os.path.join(gpath, f'networkmetrics.csv'), index=False, na_rep='NULL')