library("readxl")
library(MASS)
library(robustbase)
library(car)
library(ggplot2)
library(ggpubr)
library(olsrr)
library(sjPlot)

setwd('path/files')
d <- as.data.frame(read.csv("filename1.csv")) #file containing network metrics 
a <- as.data.frame(read.csv("filename2.csv", sep = ';')) # master table  

data <- merge(d, a, by = 'Subj')

Age_2 <- data$Age**2
data <- cbind(data, Age_2)

f <- subset(data, data$Sex=='f')
m <- subset(data, data$Sex=='m')

min(f$Age) 
max(f$Age)
quantile(f$Age)

min(m$Age) 
max(m$Age) 
quantile(m$Age)

###################################################################################################
#DENSITY
den <- lmrob(density ~ Sex + WM_volume + Age_2 + Age, data = data, setting = "KS2014")
summary(den)

newdat <- data.frame(Age = c(18:70), 
                     Sex  = 'm', 
                     WM_volume = 790112.7) 
newdat$Age_2 <- newdat$Age^2

newdat$pred <- predict(den, newdata = newdat)
p <- predict(den, newdata = newdat, se.fit = TRUE)
newdat$se.fit <- p$se.fit
newdat$LL <- newdat$pred - 1.96 * newdat$se.fit
newdat$UL <- newdat$pred + 1.96 * newdat$se.fit

ggplot(newdat, aes(x = Age, y = pred, ymin = LL, ymax = UL)) +
  geom_line(col = "blue") + 
  ylab('predicted density') + theme_bw() + geom_ribbon(alpha = 0.3, fill = "deepskyblue")

#### PREDICTION ####

data$Density_norm <- (data$density - mean(data$density))/sd(data$density)

data$group <- rep(1:5)

data$prediction_Density <- NA

for(k in c(1:5)){
  ## training
  mod_den <- lm(Density_norm ~ Sex + WM_volume + Age + Age_2, data = data[data$group != k, ])
  ## test
  data[data$group == k, "prediction.den"] <- predict(mod_den,
                                                     newdata = data[data$group == k, ]) 
  
}

data$pred.error.den <- apply(data[, c("Density_norm", "prediction.den")], 1, diff)
data[c(1:10), c("Density_norm", "prediction.den", "pred.error.den")]

summary(data$pred.error.den^2)
###################################################################################################

#EFFICIENCY

eff <- lmrob(efficiency ~ Sex + WM_volume + Age + Age_2, data = data, setting = "KS2014")
summary(eff)

newdat <- data.frame(Age = c(18:70), 
                     Sex  = 'm', 
                     WM_volume = 790112.7)
newdat$Age_2 <- newdat$Age^2

newdat$pred <- predict(eff, newdata = newdat)
p <- predict(eff, newdata = newdat, se.fit = TRUE)
newdat$se.fit <- p$se.fit
newdat$LL <- newdat$pred - 1.96 * newdat$se.fit
newdat$UL <- newdat$pred + 1.96 * newdat$se.fit

ggplot(newdat, aes(x = Age, y = pred, ymin = LL, ymax = UL)) +
  geom_line(col = "blue") + 
  ylab('predicted efficiency') + theme_bw() + geom_ribbon(alpha = 0.3, fill = "deepskyblue")

#### PREDICTION ####

data$Efficiency_norm <- (data$efficiency - mean(data$efficiency))/sd(data$efficiency)

data$prediction_Efficiency <- NA

for(k in c(1:5)){
  ## training
  mod_eff <- lm(Efficiency_norm ~ Sex + WM_volume + Age + Age_2, data = data[data$group != k, ])
  ## test
  data[data$group == k, "prediction.eff"] <- predict(mod_eff,
                                                     newdata = data[data$group == k, ]) 
  
}

data$pred.error.eff <- apply(data[, c("Efficiency_norm", "prediction.eff")], 1, diff)
data[c(1:10), c("Efficiency_norm", "prediction.eff", "pred.error.eff")]

summary(data$pred.error.eff^2)
############################################################

#MODULARITY

mod <- lmrob(modularity ~ Sex + WM_volume + Age_2 + Age, data = data, setting = "KS2014")
summary(mod)

#### PREDICTION ####

data$Modularity_norm <- (data$modularity - mean(data$modularity))/sd(data$modularity)

data$prediction_Modularity <- NA

for(k in c(1:5)){
  ## training
  mod_mod <- lm(Modularity_norm ~ Sex + WM_volume + Age + Age_2, data = data[data$group != k, ])
  ## test
  data[data$group == k, "prediction.mod"] <- predict(mod_mod,
                                                     newdata = data[data$group == k, ]) 
  
}

data$pred.error.mod <- apply(data[, c("Modularity_norm", "prediction.mod")], 1, diff)
data[c(1:10), c("Modularity_norm", "prediction.mod", "pred.error.mod")]

summary(data$pred.error.mod^2)

###################################################################################################
ms <- lmrob(strength ~ Sex + WM_volume + Age_2 + Age, data = data, setting = "KS2014")
summary(ms)

newdat <- data.frame(Age = c(18:70), 
                     Sex  = 'm', 
                     WM_volume = 790112.7)
newdat$Age_2 <- newdat$Age^2

newdat$pred <- predict(ms, newdata = newdat)
p <- predict(ms, newdata = newdat, se.fit = TRUE)
newdat$se.fit <- p$se.fit
newdat$LL <- newdat$pred - 1.96 * newdat$se.fit
newdat$UL <- newdat$pred + 1.96 * newdat$se.fit

ggplot(newdat, aes(x = Age, y = pred, ymin = LL, ymax = UL)) +
  geom_line(col = "blue") + 
  ylab('predicted strength') + theme_bw() + geom_ribbon(alpha = 0.3, fill = "deepskyblue")

#### PREDICTION ####

data$Strength_norm <- (data$strength - mean(data$strength))/sd(data$strength)

data$prediction_Strength <- NA

for(k in c(1:5)){
  ## training
  mod_ms <- lm(Strength_norm ~ Sex + WM_volume + Age + Age_2, data = data[data$group != k, ])
  ## test
  data[data$group == k, "prediction.ms"] <- predict(mod_ms,
                                                    newdata = data[data$group == k, ]) 
  
}

data$pred.error.ms <- apply(data[, c("Strength_norm", "prediction.ms")], 1, diff)
data[c(1:10), c("Strength_norm", "prediction.ms", "pred.error.ms")]

summary(data$pred.error.ms^2)

